@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/files.c,v 1.14 2009/10/22 18:17:51 amb Exp $
+ $Header: /home/amb/routino/src/RCS/files.c,v 1.18 2010/03/29 18:20:06 amb Exp $
 
- Functions to map a file into memory.
+ Functions to handle files.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -26,6 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
+#include <errno.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <sys/types.h>
@@ -74,7 +75,7 @@ char *FileName(const char *dirname,const char *prefix, const char *name)
 /*++++++++++++++++++++++++++++++++++++++
   Open a file and map it into memory.
 
-  void *MapFile Returns the address of the file.
+  void *MapFile Returns the address of the file or exits in case of an error.
 
   const char *filename The name of the file to open.
   ++++++++++++++++++++++++++++++++++++++*/
@@ -82,38 +83,24 @@ char *FileName(const char *dirname,const char *prefix, const char *name)
 void *MapFile(const char *filename)
 {
  int fd;
- struct stat buf;
+ off_t size;
  void *address;
 
- /* Open the file */
+ /* Open the file and get its size */
 
- fd=open(filename,O_RDONLY);
+ fd=ReOpenFile(filename);
 
- if(fd<0)
-   {
-    fprintf(stderr,"Cannot open file '%s' to read.\n",filename);
-    exit(EXIT_FAILURE);
-   }
-
- /* Get the length of the file */
-
- if(fstat(fd,&buf))
-   {
-    close(fd);
-
-    fprintf(stderr,"Cannot stat file '%s'.\n",filename);
-    exit(EXIT_FAILURE);
-   }
+ size=SizeFile(filename);
 
  /* Map the file */
 
- address=mmap(NULL,buf.st_size,PROT_READ,MAP_SHARED,fd,0);
+ address=mmap(NULL,size,PROT_READ,MAP_SHARED,fd,0);
 
  if(address==MAP_FAILED)
    {
     close(fd);
 
-    fprintf(stderr,"Cannot mmap file '%s'.\n",filename);
+    fprintf(stderr,"Cannot mmap file '%s' [%s].\n",filename,strerror(errno));
     exit(EXIT_FAILURE);
    }
 
@@ -122,7 +109,7 @@ void *MapFile(const char *filename)
  mappedfiles[nmappedfiles].filename=filename;
  mappedfiles[nmappedfiles].fd=fd;
  mappedfiles[nmappedfiles].address=address;
- mappedfiles[nmappedfiles].length=buf.st_size;
+ mappedfiles[nmappedfiles].length=size;
 
  nmappedfiles++;
 
@@ -131,7 +118,7 @@ void *MapFile(const char *filename)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Unmap a file and optionally delete it.
+  Unmap a file.
 
   void *UnmapFile Returns NULL (for similarity to the MapFile function).
 
@@ -148,7 +135,7 @@ void *UnmapFile(const char *filename)
 
  if(i==nmappedfiles)
    {
-    fprintf(stderr,"Cannot find file '%s' to unmap.\n",filename);
+    fprintf(stderr,"The file '%s' was not mapped using MapFile().\n",filename);
     exit(EXIT_FAILURE);
    }
 
@@ -174,7 +161,7 @@ void *UnmapFile(const char *filename)
 /*++++++++++++++++++++++++++++++++++++++
   Open a new file on disk for writing to.
 
-  int OpenFile Returns the file descriptor if OK or something negative in case of an error.
+  int OpenFile Returns the file descriptor if OK or exits in case of an error.
 
   const char *filename The name of the file to create.
   ++++++++++++++++++++++++++++++++++++++*/
@@ -189,7 +176,33 @@ int OpenFile(const char *filename)
 
  if(fd<0)
    {
-    fprintf(stderr,"Cannot open file '%s' to write.\n",filename);
+    fprintf(stderr,"Cannot open file '%s' for writing [%s].\n",filename,strerror(errno));
+    exit(EXIT_FAILURE);
+   }
+
+ return(fd);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Open a new file on disk for reading and appending.
+
+  int AppendFile Returns the file descriptor if OK or exits in case of an error.
+
+  const char *filename The name of the file to create.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int AppendFile(const char *filename)
+{
+ int fd;
+
+ /* Open the file */
+
+ fd=open(filename,O_RDWR|O_CREAT|O_APPEND,S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
+
+ if(fd<0)
+   {
+    fprintf(stderr,"Cannot open file '%s' for appending [%s].\n",filename,strerror(errno));
     exit(EXIT_FAILURE);
    }
 
@@ -200,7 +213,7 @@ int OpenFile(const char *filename)
 /*++++++++++++++++++++++++++++++++++++++
   Open an existing file on disk for reading from.
 
-  int ReOpenFile Returns the file descriptor if OK or something negative in case of an error.
+  int ReOpenFile Returns the file descriptor if OK or exits in case of an error.
 
   const char *filename The name of the file to open.
   ++++++++++++++++++++++++++++++++++++++*/
@@ -215,7 +228,7 @@ int ReOpenFile(const char *filename)
 
  if(fd<0)
    {
-    fprintf(stderr,"Cannot open file '%s' to read.\n",filename);
+    fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",filename,strerror(errno));
     exit(EXIT_FAILURE);
    }
 
@@ -269,6 +282,47 @@ int ReadFile(int fd,void *address,size_t length)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Get the size of a file.
+
+  off_t SizeFile Returns the file size.
+
+  const char *filename The name of the file to check.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+off_t SizeFile(const char *filename)
+{
+ struct stat buf;
+
+ if(stat(filename,&buf))
+   {
+    fprintf(stderr,"Cannot stat file '%s' [%s].\n",filename,strerror(errno));
+    exit(EXIT_FAILURE);
+   }
+
+ return(buf.st_size);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Check if a file exists.
+
+  int ExistsFile Returns 1 if the file exists and 0 if not.
+
+  const char *filename The name of the file to check.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int ExistsFile(const char *filename)
+{
+ struct stat buf;
+
+ if(stat(filename,&buf))
+    return(0);
+ else
+    return(1);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Seek to a position in a file on disk.
 
@@ -276,10 +330,10 @@ int ReadFile(int fd,void *address,size_t length)
 
   int fd The file descriptor to seek within.
 
-  size_t position The position to seek to.
+  off_t position The position to seek to.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int SeekFile(int fd,size_t position)
+int SeekFile(int fd,off_t position)
 {
  /* Seek the data */
 
