@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/functions.h,v 1.48 2010/01/15 19:48:39 amb Exp $
+ $Header: /home/amb/routino/src/RCS/functions.h,v 1.54 2010/04/24 16:47:56 amb Exp $
 
  Header file for function prototypes
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -25,6 +25,7 @@
 #ifndef FUNCTIONS_H
 #define FUNCTIONS_H    /*+ To stop multiple inclusions. +*/
 
+#include <sys/types.h>
 #include <stdio.h>
 
 #include "types.h"
@@ -54,11 +55,16 @@ void *MapFile(const char *filename);
 void *UnmapFile(const char *filename);
 
 int OpenFile(const char *filename);
+int AppendFile(const char *filename);
 int ReOpenFile(const char *filename);
 
 int WriteFile(int fd,const void *address,size_t length);
 int ReadFile(int fd,void *address,size_t length);
-int SeekFile(int fd,size_t position);
+
+off_t SizeFile(const char *filename);
+int ExistsFile(const char *filename);
+
+int SeekFile(int fd,off_t position);
 
 void CloseFile(int fd);
 
@@ -80,16 +86,15 @@ void FixForwardRoute(Results *results,index_t finish);
 
 /* In output.c */
 
-void PrintRouteHead(const char *copyright);
 void PrintRoute(Results **results,int nresults,Nodes *nodes,Segments *segments,Ways *ways,Profile *profile);
-void PrintRouteTail(void);
 
 
 /* In sorting.c */
 
-/*+ The type and size of variable to store the variable length +*/
+/*+ The type, size and alignment of variable to store the variable length +*/
 #define FILESORT_VARINT   unsigned short
 #define FILESORT_VARSIZE  sizeof(FILESORT_VARINT)
+#define FILESORT_VARALIGN sizeof(void*)
 
 void filesort_fixed(int fd_in,int fd_out,size_t itemsize,int (*compare)(const void*,const void*),int (*buildindex)(void*,index_t));
 
