@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/segments.c,v 1.43 2009/07/12 09:01:48 amb Exp $
+ $Header: /home/amb/routino/src/RCS/segments.c,v 1.45 2010/04/28 17:27:02 amb Exp $
 
  Segment data type functions.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -50,9 +50,6 @@ Segments *LoadSegmentList(const char *filename)
 
  data=MapFile(filename);
 
- if(!data)
-    return(NULL);
-
  /* Copy the Segments structure from the loaded data */
 
  *segments=*((Segments*)data);
@@ -60,7 +57,7 @@ Segments *LoadSegmentList(const char *filename)
  /* Adjust the pointers in the Segments structure. */
 
  segments->data=data;
- segments->segments=(Segment*)(data+(off_t)segments->segments);
+ segments->segments=(Segment*)(data+sizeof(Segments));
 
  return(segments);
 }
