@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/translations.c,v 1.8 2010/05/29 13:54:23 amb Exp $
+ $Header: /home/amb/routino/src/RCS/translations.c,v 1.10 2010/07/03 10:58:37 amb Exp $
 
  Load the translations from a file and the functions for handling them.
 
@@ -33,7 +33,7 @@
 
 /* Global variables - default English values */
 
-char *translate_copyright_creator[2]={"Creator","Routino - http://www.gedanken.org.uk/mapping/routino/"};
+char *translate_copyright_creator[2]={"Creator","Routino - http://www.routino.org/"};
 char *translate_copyright_source[2] ={NULL,NULL};
 char *translate_copyright_license[2]={NULL,NULL};
 
@@ -952,7 +952,7 @@ static int languageType_function(const char *_tag_,int _type_,const char *lang)
 
     if(!store_lang && first)
        store=1;
-    else if(!strcmp(store_lang,lang))
+    else if(store_lang && !strcmp(store_lang,lang))
        store=1;
     else
        store=0;
