@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/types.h,v 1.38 2009/12/15 18:44:28 amb Exp $
+ $Header: /home/amb/routino/src/RCS/types.h,v 1.40 2010/05/27 17:25:23 amb Exp $
 
  Type definitions
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -179,7 +179,7 @@ typedef enum _Highway
   Way_Path        =11,
   Way_Steps       =12,
 
-  Way_Count       =13,       /* One more than the number of highways. */
+  Way_Count       =13,       /* One more than the number of highway types. */
 
   Way_OneWay      =32,
   Way_Roundabout  =64
@@ -203,7 +203,9 @@ typedef enum _Transport
   Transport_Motorcar   =  7,
   Transport_Goods      =  8,
   Transport_HGV        =  9,
-  Transport_PSV        = 10
+  Transport_PSV        = 10,
+
+  Transport_Count      = 11     /*+ One more than the number of transport types. +*/
  }
  Transport;
 
@@ -242,7 +244,7 @@ typedef enum _Property
   Property_Bridge    = 3,
   Property_Tunnel    = 4,
 
-  Property_Count     = 5       /* One more than the number of properties. */
+  Property_Count     = 5       /* One more than the number of property types. */
  }
  Property;
 
@@ -327,4 +329,22 @@ typedef struct _Way Way;
 typedef struct _Ways Ways;
 
 
+/* Functions */
+
+Highway HighwayType(const char *highway);
+Transport TransportType(const char *transport);
+Property PropertyType(const char *property);
+
+const char *HighwayName(Highway highway);
+const char *TransportName(Transport transport);
+const char *PropertyName(Property property);
+
+const char *AllowedNameList(wayallow_t allowed);
+const char *PropertiesNameList(wayprop_t properties);
+
+const char *HighwayList(void);
+const char *TransportList(void);
+const char *PropertyList(void);
+
+
 #endif /* TYPES_H */
