@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/ways.h,v 1.34 2009/11/25 15:00:37 amb Exp $
+ $Header: /home/amb/routino/src/RCS/ways.h,v 1.37 2010/05/29 13:54:24 amb Exp $
 
  A header file for the ways.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -59,8 +59,8 @@ struct _Ways
  uint32_t   number;             /*+ How many ways are stored? +*/
  uint32_t   onumber;            /*+ How many ways were there originally? +*/
 
- wayallow_t allow;              /*+ The type of traffic allowed when parsing. +*/
- wayprop_t  props;              /*+ The properties that were included when parsing. +*/
+ wayallow_t allow;              /*+ The types of traffic that were seen when parsing. +*/
+ wayprop_t  props;              /*+ The properties that were seen when parsing. +*/
 
  Way       *ways;               /*+ An array of ways. +*/
  char      *names;              /*+ An array of characters containing the names. +*/
@@ -75,8 +75,14 @@ struct _Ways
 /*+ Return a Way* pointer given a set of ways and an index. +*/
 #define LookupWay(xxx,yyy)     (&(xxx)->ways[yyy])
 
-/*+ Return the name of a way given the Way pointer and a set of ways. +*/
-#define WayName(xxx,yyy)       (&(xxx)->names[(yyy)->name])
+/*+ Return the raw name of a way given the Way pointer and a set of ways. +*/
+#define WayNameRaw(xxx,yyy)        (&(xxx)->names[(yyy)->name])
+
+/*+ Decide if a way has a name or not. +*/
+#define WayNamed(xxx,yyy)          ((xxx)->names[(yyy)->name])
+
+/*+ Return the name of a way if it has one or the name of the highway type otherwise. +*/
+#define WayNameHighway(xxx,yyy)    (WayNamed(xxx,yyy)?WayNameRaw(xxx,yyy):HighwayName(HIGHWAY(yyy->type)))
 
 
 /* Functions */
@@ -84,21 +90,6 @@ struct _Ways
 
 Ways *LoadWayList(const char *filename);
 
-Highway HighwayType(const char *highway);
-Transport TransportType(const char *transport);
-Property PropertyType(const char *property);
-
-const char *HighwayName(Highway highway);
-const char *TransportName(Transport transport);
-const char *PropertyName(Property property);
-
-const char *AllowedNameList(wayallow_t allowed);
-const char *PropertiesNameList(wayprop_t properties);
-
-const char *HighwayList(void);
-const char *TransportList(void);
-const char *PropertyList(void);
-
 int WaysCompare(Way *way1,Way *way2);
 
 
