@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/waysx.c,v 1.34 2009/12/12 11:08:50 amb Exp $
+ $Header: /home/amb/routino/src/RCS/waysx.c,v 1.38 2010/05/22 18:40:47 amb Exp $
 
  Extended Way data type functions.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -25,6 +25,7 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
 
 #include "functions.h"
 #include "waysx.h"
@@ -52,12 +53,14 @@ static int index_by_id(WayX *wayx,index_t index);
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Allocate a new way list.
+  Allocate a new way list (create a new file or open an existing one).
 
   WaysX *NewWayList Returns the way list.
+
+  int append Set to 1 if the file is to be opened for appending (now or later).
   ++++++++++++++++++++++++++++++++++++++*/
 
-WaysX *NewWayList(void)
+WaysX *NewWayList(int append)
 {
  WaysX *waysx;
 
@@ -66,9 +69,35 @@ WaysX *NewWayList(void)
  assert(waysx); /* Check calloc() worked */
 
  waysx->filename=(char*)malloc(strlen(option_tmpdirname)+32);
- sprintf(waysx->filename,"%s/ways.%p.tmp",option_tmpdirname,waysx);
 
- waysx->fd=OpenFile(waysx->filename);
+ if(append)
+    sprintf(waysx->filename,"%s/ways.input.tmp",option_tmpdirname);
+ else
+    sprintf(waysx->filename,"%s/ways.%p.tmp",option_tmpdirname,waysx);
+
+ if(append)
+   {
+    off_t size,position=0;
+
+    waysx->fd=AppendFile(waysx->filename);
+
+    size=SizeFile(waysx->filename);
+
+    while(position<size)
+      {
+       FILESORT_VARINT waysize;
+
+       SeekFile(waysx->fd,position);
+       ReadFile(waysx->fd,&waysize,FILESORT_VARSIZE);
+
+       waysx->xnumber++;
+       position+=waysize+FILESORT_VARSIZE;
+      }
+
+    SeekFile(waysx->fd,size);
+   }
+ else
+    waysx->fd=OpenFile(waysx->filename);
 
  waysx->nfilename=(char*)malloc(strlen(option_tmpdirname)+32);
  sprintf(waysx->nfilename,"%s/waynames.%p.tmp",option_tmpdirname,waysx);
@@ -81,17 +110,22 @@ WaysX *NewWayList(void)
   Free a way list.
 
   WaysX *waysx The list to be freed.
+
+  int keep Set to 1 if the file is to be kept.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void FreeWayList(WaysX *waysx)
+void FreeWayList(WaysX *waysx,int keep)
 {
- DeleteFile(waysx->filename);
+ if(!keep)
+    DeleteFile(waysx->filename);
+
  free(waysx->filename);
 
  if(waysx->idata)
     free(waysx->idata);
 
  DeleteFile(waysx->nfilename);
+
  free(waysx->nfilename);
 
  free(waysx);
@@ -499,11 +533,9 @@ WayX *LookupWayX(WaysX* waysx,index_t index,int position)
   WaysX* waysx The set of ways to save.
 
   const char *filename The name of the file to save.
-
-  Profile *profile The profile used during parsing.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void SaveWayList(WaysX* waysx,const char *filename,Profile *profile)
+void SaveWayList(WaysX* waysx,const char *filename)
 {
  index_t i;
  int fd,nfd;
@@ -526,28 +558,24 @@ void SaveWayList(WaysX* waysx,const char *filename,Profile *profile)
  ways->number=waysx->cnumber;
  ways->onumber=waysx->number;
 
- ways->allow=profile->allow;
-
+ ways->allow=0;
  ways->props=0;
- for(i=1;i<Property_Count;i++)
-    if(profile->props_yes[i])
-       ways->props|=PROPERTIES(i);
 
  ways->data=NULL;
-
- ways->ways=(void*)sizeof(Ways);
- ways->names=(void*)(sizeof(Ways)+ways->number*sizeof(Way));
+ ways->ways=NULL;
+ ways->names=NULL;
 
  /* Write out the Ways structure and then the real data. */
 
  fd=OpenFile(filename);
 
- WriteFile(fd,ways,sizeof(Ways));
-
  for(i=0;i<waysx->number;i++)
    {
     WayX *wayx=LookupWayX(waysx,i,1);
 
+    ways->allow|=wayx->way.allow;
+    ways->props|=wayx->way.props;
+
     SeekFile(fd,sizeof(Ways)+wayx->prop*sizeof(Way));
     WriteFile(fd,&wayx->way,sizeof(Way));
 
@@ -558,6 +586,9 @@ void SaveWayList(WaysX* waysx,const char *filename,Profile *profile)
       }
    }
 
+ SeekFile(fd,0);
+ WriteFile(fd,ways,sizeof(Ways));
+
  if(!option_slim)
     waysx->xdata=UnmapFile(waysx->filename);
 
