@@ -1,11 +1,11 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/waysx.h,v 1.19 2009/12/11 19:27:39 amb Exp $
+ $Header: /home/amb/routino/src/RCS/waysx.h,v 1.21 2010/05/22 18:40:47 amb Exp $
 
  A header file for the extended Ways structure.
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008,2009 Andrew M. Bishop
+ This file Copyright 2008-2010 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -30,7 +30,6 @@
 #include "typesx.h"
 #include "types.h"
 #include "ways.h"
-#include "profiles.h"
 
 
 /* Data structures */
@@ -73,10 +72,10 @@ struct _WaysX
 /* Functions */
 
 
-WaysX *NewWayList(void);
-void FreeWayList(WaysX *waysx);
+WaysX *NewWayList(int append);
+void FreeWayList(WaysX *waysx,int keep);
 
-void SaveWayList(WaysX *waysx,const char *filename,Profile *profile);
+void SaveWayList(WaysX *waysx,const char *filename);
 
 index_t IndexWayX(WaysX* waysx,way_t id);
 WayX *LookupWayX(WaysX* waysx,index_t index,int position);
